class PublishedFlow {
  final String? id;
  final String? workspaceID;
  final String? name;
  final String? userID;
  final String? flowID;
  final String? versionID;
  final Map? labels;
  final List<VersionElement>? versions;
  final DateTime? createdAt;
  final DateTime? updatedAt;

  PublishedFlow({
    this.id,
    this.workspaceID,
    this.name,
    this.userID,
    this.flowID,
    this.versionID,
    this.labels,
    this.versions,
    this.createdAt,
    this.updatedAt,
  });

  factory PublishedFlow.fromJson(Map<dynamic, dynamic> json) => PublishedFlow(
        id: json["id"],
        workspaceID: json["workspace_id"],
        name: json["name"],
        userID: json["publisher_id"],
        flowID: json["flow_id"],
        versionID: json["version_id"],
        labels: json["labels"],
        versions: (json["versions"] as List)
            .map(
              (e) => VersionElement.fromJson(e),
            )
            .toList(),
        createdAt: DateTime.parse(json["created_at"]),
        updatedAt: DateTime.parse(json["updated_at"]),
      );

  Map<String, dynamic> toJson() => {
        "id": id,
        "workspace_id": workspaceID,
        "name": name,
        "publisher_id": userID,
        "flow_id": flowID,
        "version_id": versionID,
        "labels": labels,
        "versions": versions,
        "created_at": createdAt.toString(),
        "updated_at": updatedAt.toString(),
      };
}

class VersionElement {
  final String? publishedFlowID;
  final bool? scheduled;
  final String? versionID;
  final String? versionName;

  VersionElement({
    this.publishedFlowID,
    this.scheduled,
    this.versionID,
    this.versionName,
  });

  factory VersionElement.fromJson(Map<dynamic, dynamic> json) => VersionElement(
        publishedFlowID: json["published_flow_id"],
        scheduled: json["scheduled"],
        versionID: json["version_id"],
        versionName: json["version_name"],
      );

  Map<String, dynamic> toJson() => {
        "published_flow_id": publishedFlowID,
        "scheduled": scheduled,
        "version_id": versionID,
        "version_name": versionName,
      };
}
