import 'dart:convert';
import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:robomotion_app/robomotion_app.dart';
import 'package:version/version.dart';

class QueueMessage {
  String? id;
  String? appID;
  String? robotID;
  String? userID;
  int? msgType;
  Uint8List? payload;
  bool? encrypted;
  Version? version;
  Robomotion? robomotion;

  QueueMessage({
    required this.id,
    required this.appID,
    this.robotID,
    this.userID,
    this.msgType = QMT_KeyExchange,
    this.payload,
    this.encrypted,
    this.version,
  });

  factory QueueMessage.fromJson(Map<dynamic, dynamic> json) => QueueMessage(
        id: json["id"],
        appID: json["app_id"],
        robotID: json["robot_id"],
        userID: json["user_id"],
        msgType: json["msg_type"],
        payload: base64Decode(json["payload"]),
        encrypted: json["encrypted"],
        version:
            json["version"] == null ? null : Version.parse(json["version"]),
      );

  Map<String, dynamic> toJson() => {
        "id": id,
        "app_id": appID,
        "robot_id": robotID,
        "user_id": userID,
        "msg_type": msgType,
        "payload": payload != null ? base64Encode(payload!) : null,
        "encrypted": encrypted,
        "version": version?.toString(),
      };

  @override
  String toString() {
    return jsonEncode(toJson());
  }

  Map<String, dynamic>? get data {
    if (payload == null) {
      return null;
    }
    try {
      String decodedString = utf8.decode(payload!);
      return jsonDecode(decodedString);
    } catch (e) {
      print("Error decoding payload: $e");
      return null;
    }
  }

  void replyBytes(Uint8List data) {
    _validateForReply();
    this.payload = data;
    this.robomotion?.sendAppMsg(this);
  }

  void reply(Map<String, dynamic> data) {
    _validateForReply();
    try {
      String jsonString = jsonEncode(data);
      this.payload = Uint8List.fromList(utf8.encode(jsonString));
      this.robomotion!.sendAppMsg(this);
    } catch (e) {
      print("Error encoding JSON payload: $e");
      throw Exception("Failed to encode JSON payload");
    }
  }

  void _validateForReply() {
    if (this.msgType != QMT_AppRequest) {
      throw Exception("Cannot reply to a non-request message");
    }
    if (this.robomotion == null) {
      throw Exception("Cannot reply to a message without robomotion instance");
    }
  }
}

const int QMT_Error = -1;
const int QMT_KeyExchange = 0;
const int QMT_AppIn = 1;
const int QMT_AppOut = 2;
const int QMT_AppPush = 3;
const int QMT_AppRequest = 4;
const int QMT_RobotID = 5;
